@@ -1,7 +1,7 @@
 package OtherRoleWithAttributes;
-our $VERSION = '0.16_01';
+our $VERSION = '0.15';
 
-use MooseX::MethodAttributes::Role;
+use Moose::Role -traits => 'MethodAttributes';
 use namespace::clean -except => 'meta';
 
 sub bar : AnAttr { 'bar' }
