@@ -1,5 +1,5 @@
 package UsesMultipleRoles;
-our $VERSION = '0.16_01';
+our $VERSION = '0.15';
 
 use Moose;
 use namespace::clean -except => 'meta';
@@ -9,5 +9,5 @@ with qw/
     OtherRoleWithAttributes
 /;
 
-#__PACKAGE__->meta->make_immutable;
+__PACKAGE__->meta->make_immutable;
 
